import { AttributeType, gl } from '@antv/l7-core';
import BaseModel from '../../core/BaseModel';
import { PolygonExtrudeTriangulation } from '../../core/triangulation';
const polygonExtrudeFrag = "uniform float u_opacity: 1.0;\nvarying vec4 v_Color;\n\n#pragma include \"picking\"\n\nvoid main() {\n  gl_FragColor = v_Color;\n  gl_FragColor.a *= u_opacity;\n  gl_FragColor = filterColor(gl_FragColor);\n}\n";
const polygonExtrudeVert = "precision highp float;\n\n#define ambientRatio 0.5\n#define diffuseRatio 0.3\n#define specularRatio 0.2\n\nattribute vec4 a_Color;\nattribute vec3 a_Position;\nattribute vec3 a_Normal;\nattribute float a_Size;\nuniform mat4 u_ModelMatrix;\n\nvarying vec4 v_Color;\n\n#pragma include \"projection\"\n#pragma include \"picking\"\n\nvoid main() {\n  vec4 project_pos = project_position(vec4(a_Position.xy, project_pixel(a_Position.z * a_Size), 1.0));\n\n  gl_Position = project_common_position_to_clipspace(vec4(project_pos.xyz, 1.0));\n\n  vec3 worldPos = vec3(project_pos * u_ModelMatrix);\n\n  vec3 worldNormal = vec3(vec4(a_Normal,1.0) * u_ModelMatrix);\n  //  vec3 worldNormal = project_normal(a_Normal);\n  // //cal light weight\n  vec3 viewDir = normalize(u_CameraPosition - worldPos);\n\n  vec3 lightDir = normalize(vec3(1, -10.5, 12));\n\n  vec3 halfDir = normalize(viewDir+lightDir);\n  // //lambert\n  float lambert = dot(worldNormal, lightDir);\n  //specular\n  float specular = pow( max(0.0, dot(worldNormal, halfDir)), 16.0);\n  //sum to light weight\n  float lightWeight = ambientRatio + diffuseRatio * lambert + specularRatio * specular;\n  // v_Color = a_Color;\n  v_Color = vec4(a_Color.rgb * lightWeight, a_Color.w); \n\n  setPickingColor(a_PickingColor);\n}";
export default class ExtrudeModel extends BaseModel {
  getUninforms() {
    const {
      opacity = 1
    } = this.layer.getLayerConfig();
    return {
      u_opacity: opacity
    };
  }

  buildModels() {
    return [this.layer.buildLayerModel({
      moduleName: 'polygonExtrude',
      vertexShader: polygonExtrudeVert,
      fragmentShader: polygonExtrudeFrag,
      triangulation: PolygonExtrudeTriangulation
    })];
  }

  registerBuiltinAttributes() {
    this.styleAttributeService.registerStyleAttribute({
      name: 'normal',
      type: AttributeType.Attribute,
      descriptor: {
        name: 'a_Normal',
        buffer: {
          usage: gl.STATIC_DRAW,
          data: [],
          type: gl.FLOAT
        },
        size: 3,
        update: (feature, featureIdx, vertex, attributeIdx, normal) => {
          return normal;
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: 'size',
      type: AttributeType.Attribute,
      descriptor: {
        name: 'a_Size',
        buffer: {
          usage: gl.DYNAMIC_DRAW,
          data: [],
          type: gl.FLOAT
        },
        size: 1,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          const {
            size
          } = feature;
          return Array.isArray(size) ? [size[0]] : [size];
        }
      }
    });
  }

}
//# sourceMappingURL=extrude.js.map