"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _l7Core = require("@antv/l7-core");

var _BaseLayer = _interopRequireDefault(require("../core/BaseLayer"));

var _triangulation = require("../core/triangulation");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const polygonExtrudeFrag = "uniform float u_opacity: 1.0;\nvarying vec4 v_Color;\n\n#pragma include \"picking\"\n\nvoid main() {\n  gl_FragColor = v_Color;\n  gl_FragColor.a *= u_opacity;\n  gl_FragColor = filterColor(gl_FragColor);\n}\n";
const polygonExtrudeVert = "precision highp float;\n\n#define ambientRatio 0.5\n#define diffuseRatio 0.3\n#define specularRatio 0.2\n\nattribute vec4 a_Color;\nattribute vec3 a_Position;\nattribute vec3 a_Normal;\nattribute float a_Size;\nuniform mat4 u_ModelMatrix;\n\nvarying vec4 v_Color;\n\n#pragma include \"projection\"\n#pragma include \"picking\"\n\nvoid main() {\n  vec4 project_pos = project_position(vec4(a_Position.xy, project_pixel(a_Position.z * a_Size), 1.0));\n\n  gl_Position = project_common_position_to_clipspace(vec4(project_pos.xyz, 1.0));\n\n  vec3 worldPos = vec3(project_pos * u_ModelMatrix);\n\n  vec3 worldNormal = vec3(vec4(a_Normal,1.0) * u_ModelMatrix);\n  //  vec3 worldNormal = project_normal(a_Normal);\n  // //cal light weight\n  vec3 viewDir = normalize(u_CameraPosition - worldPos);\n\n  vec3 lightDir = normalize(vec3(1, -10.5, 12));\n\n  vec3 halfDir = normalize(viewDir+lightDir);\n  // //lambert\n  float lambert = dot(worldNormal, lightDir);\n  //specular\n  float specular = pow( max(0.0, dot(worldNormal, halfDir)), 16.0);\n  //sum to light weight\n  float lightWeight = ambientRatio + diffuseRatio * lambert + specularRatio * specular;\n  // v_Color = a_Color;\n  v_Color = vec4(a_Color.rgb * lightWeight, a_Color.w); \n\n  setPickingColor(a_PickingColor);\n}";

class PolygonLayer extends _BaseLayer.default {
  constructor(...args) {
    super(...args);
    this.name = 'PolygonLayer';
  }

  getConfigSchema() {
    return {
      properties: {
        opacity: {
          type: 'number',
          minimum: 0,
          maximum: 1
        }
      }
    };
  }

  renderModels() {
    const {
      opacity
    } = this.getLayerConfig();
    this.models.forEach(model => model.draw({
      uniforms: {
        u_opacity: opacity || 1.0
      }
    }));
    return this;
  }

  buildModels() {
    this.registerBuiltinAttributes();
    this.models = [this.buildLayerModel({
      moduleName: 'polygonExtrude',
      vertexShader: polygonExtrudeVert,
      fragmentShader: polygonExtrudeFrag,
      triangulation: _triangulation.PolygonExtrudeTriangulation
    })];
  }

  registerBuiltinAttributes() {
    this.styleAttributeService.registerStyleAttribute({
      name: 'normal',
      type: _l7Core.AttributeType.Attribute,
      descriptor: {
        name: 'a_Normal',
        buffer: {
          usage: _l7Core.gl.STATIC_DRAW,
          data: [],
          type: _l7Core.gl.FLOAT
        },
        size: 3,
        update: (feature, featureIdx, vertex, attributeIdx, normal) => {
          return normal;
        }
      }
    });
    this.styleAttributeService.registerStyleAttribute({
      name: 'size',
      type: _l7Core.AttributeType.Attribute,
      descriptor: {
        name: 'a_Size',
        buffer: {
          usage: _l7Core.gl.DYNAMIC_DRAW,
          data: [],
          type: _l7Core.gl.FLOAT
        },
        size: 1,
        update: (feature, featureIdx, vertex, attributeIdx) => {
          const {
            size
          } = feature;
          return Array.isArray(size) ? [size[0]] : [size];
        }
      }
    });
  }

}

exports.default = PolygonLayer;
//# sourceMappingURL=polygon3D.js.map